import objectdraw.*;
import java.awt.*;
import java.text.DecimalFormat;
import java.util.Random;

/*
 * 
 * A simple active object that controls a ball that bounces around the
 * canvas and interacts with a boundary and a pong paddle, plus a
 * nudge method that allows someone to make a minor change to its
 * trajectory.
 *
 * Jim Teresco, Siena College, CSIS 120, Spring 2011
 * The College of Saint Rose, Fall 2013
 *
 * $Id: PongBall.java 2218 2013-10-18 14:06:39Z terescoj $
 */

public class PongBall extends ActiveObject {

    // size and speed of falling balls
    private static final int BALLSIZE = 30;
    private static final double MIN_SPEED = 3;
    private static final double MAX_SPEED = 10;
    private static final int DELAY_TIME = 33;
    private static final double NUDGE_MAX = 2;

    // the ball controlled by this instance
    private FilledOval ball;

    // the paddle with which we will interact
    private FilledRect paddle;

    // how far to fall before stopping and disappearing?
    private double yMax;
    // how far up to go before bouncing off the ceiling?
    private double yMin;
    // and what about the walls?
    private double xMin, xMax;

    // speeds are now instance variables since 
    private double xSpeed, ySpeed;

    // label for speeds
    private Text speeds;
    
    // random generator
    Random randGen = new Random();
    
    // for decimal formatting
    DecimalFormat df = new DecimalFormat("#.###");

    // Draw a ball and start it falling.
    public PongBall(FilledRect paddle, FramedRect boundary,
    DrawingCanvas aCanvas) {

        // compute/remember our boundaries (of the upper left corner of a ball)
        yMax = boundary.getY()+boundary.getHeight();
        yMin = boundary.getY();
        xMin = boundary.getX();
        xMax = boundary.getX() + boundary.getWidth() - BALLSIZE;


        // draw the ball at its initial position, which is at the 
        // top of the playing area, at a randomly-selected position
        ball = new FilledOval(randGen.nextDouble() * (xMax - xMin) + xMin,
            yMin+1, BALLSIZE, BALLSIZE, aCanvas);

        // remember the paddle
        this.paddle = paddle;
        
        // A label for the speeds
        speeds = new Text("", 10, 10, aCanvas);

        // activate!
        start();
    }

    // our "nudge" method -- make a minor change in the trajectory
    // and temporarily make the ball red
    public void nudge() {
        
        // randomly perturb both the x and y speeds
        xSpeed = xSpeed + randGen.nextDouble() * (2*NUDGE_MAX) - NUDGE_MAX;
        ySpeed = ySpeed + randGen.nextDouble() * (2*NUDGE_MAX) - NUDGE_MAX;

        speeds.setText("xSpeed = " + df.format(xSpeed) + 
                       ", ySpeed = " + df.format(ySpeed));
        
        ball.setColor(Color.red);
    }

    // move the ball repeatedly until it falls off screen, bouncing
    // off the paddle and walls along the way
    public void run() {

        // start by moving downward and some amount to the left or right
        xSpeed = randGen.nextDouble() * (MAX_SPEED - MIN_SPEED) + MIN_SPEED;
        double initYSpeed = randGen.nextDouble() * (MAX_SPEED - MIN_SPEED) + MIN_SPEED;

        ySpeed = initYSpeed;
        
        // keep moving as long as we haven't fallen off the bottom of
        // the screen
        while (ball.getY() <= yMax) {
            // if we are above the top line, start moving down
            if (ball.getY() < yMin) {
                ySpeed = initYSpeed;
            }

            // if we are in contact with the paddle, start moving up
            if (ball.overlaps(paddle)) {
                ySpeed = -initYSpeed;
            }

            // bounce off side walls
            if (ball.getX() < xMin || ball.getX() > xMax) {
                xSpeed = -xSpeed;
            }

            // move a little in the appropriate direction
            speeds.setText("xSpeed = " + df.format(xSpeed) + 
                           ", ySpeed = " + df.format(ySpeed));
            ball.move(xSpeed, ySpeed);
            speeds.moveTo(ball.getX() + BALLSIZE, ball.getY() + BALLSIZE);
            ball.setColor(Color.black);
            pause(DELAY_TIME);
        }

        ball.removeFromCanvas();
        speeds.removeFromCanvas();
    }
}
